@@ -1,12 +1,11 @@
 import re as regex
 
-from elody.policies.helpers import get_item
+from elody.policies.helpers import get_content, get_item
 from elody.policies.permission_handler import (
     get_permissions,
     handle_single_item_request,
 )
-from flask import Request  # pyright: ignore
-from flask_restful import abort  # pyright: ignore
+from flask import g, Request  # pyright: ignore
 from inuits_policy_based_auth import BaseAuthorizationPolicy  # pyright: ignore
 from inuits_policy_based_auth.contexts.policy_context import (  # pyright: ignore
     PolicyContext,
@@ -41,10 +40,10 @@ class GenericObjectDetailPolicy(BaseAuthorizationPolicy):
             access_verdict = None
             for rule in rules:
                 access_verdict = rule().apply(item, user_context, request, permissions)
-                if access_verdict != None:
+                if access_verdict is not None:
                     policy_context.access_verdict = access_verdict
                     if not policy_context.access_verdict:
-                        return policy_context
+                        break
 
             if policy_context.access_verdict:
                 return policy_context
@@ -79,8 +78,10 @@ class PutRequestRules:
         if request.method != "PUT":
             return None
 
+        content = g.get("content") or request.json
+        content = get_content(item, request, content)
         return handle_single_item_request(
-            user_context, item, permissions, "update", request.json  # pyright: ignore
+            user_context, item, permissions, "update", content
         )
 
 
@@ -91,8 +92,10 @@ class PatchRequestRules:
         if request.method != "PATCH":
             return None
 
+        content = g.get("content") or request.json
+        content = get_content(item, request, content)
         return handle_single_item_request(
-            user_context, item, permissions, "update", request.json  # pyright: ignore
+            user_context, item, permissions, "update", content
         )
 
 
