@@ -2,26 +2,29 @@ import re as regex
 
 from configuration import get_object_configuration_mapper  # pyright: ignore
 from copy import deepcopy
+from elody.error_codes import ErrorCode, get_error_code, get_read
 from elody.util import flatten_dict, interpret_flat_key
 from inuits_policy_based_auth.contexts.user_context import UserContext
 from logging_elody.log import log  # pyright: ignore
 
 
 _permissions = {}
+_placeholders = ["X_TENANT_ID", "TENANT_DEFINING_ENTITY_ID"]
 
 
-def set_permissions(permissions: dict):
+def set_permissions(permissions: dict, placeholders: list[str] = []):
     global _permissions
     _permissions = permissions
+    _placeholders.extend(placeholders)
 
 
 def get_permissions(role: str, user_context: UserContext):
     permissions = deepcopy(_permissions)
-    placeholders = ["X_TENANT_ID", "TENANT_DEFINING_ENTITY_ID"]
 
-    for placeholder in placeholders:
+    for placeholder_key in _placeholders:
+        placeholder_value = user_context.bag.get(placeholder_key.lower())
         permissions = __replace_permission_placeholders(
-            permissions, placeholder, user_context.bag[placeholder.lower()]
+            permissions, placeholder_key, placeholder_value
         )
     return permissions.get(role, {})  # pyright: ignore
 
@@ -38,7 +41,10 @@ def __replace_permission_placeholders(data, placeholder_key, placeholder_value):
             for item in data
         ]
     elif isinstance(data, str):
-        data = data.replace(placeholder_key, placeholder_value)
+        if isinstance(placeholder_value, str):
+            data = data.replace(placeholder_key, placeholder_value)
+        elif isinstance(placeholder_value, list) and data == placeholder_key:
+            data = placeholder_value
     return data
 
 
@@ -79,8 +85,8 @@ def handle_single_item_request(
 
 def mask_protected_content_post_request_hook(user_context: UserContext, permissions):
     def __post_request_hook(response):
-        items = response["results"]
-        for item in items:
+        items = []
+        for item in response["results"]:
             try:
                 (
                     item_in_storage_format,
@@ -99,6 +105,7 @@ def mask_protected_content_post_request_hook(user_context: UserContext, permissi
                     "read",
                     object_lists,
                 )
+                items.append(user_context.bag["requested_item"])
             except Exception as exception:
                 log.debug(
                     f"{exception.__class__.__name__}: {str(exception)}",
@@ -106,6 +113,7 @@ def mask_protected_content_post_request_hook(user_context: UserContext, permissi
                 )
                 raise exception
 
+        response["results"] = items
         return response
 
     return __post_request_hook
@@ -117,7 +125,7 @@ def __prepare_item_for_permission_check(item, permissions, crud):
         return item, None, None, None
 
     config = get_object_configuration_mapper().get(item["type"])
-    object_lists = config.document_info()["object_lists"]
+    object_lists = config.document_info().get("object_lists", {})
     flat_item = flatten_dict(object_lists, item)
 
     return (
@@ -167,7 +175,7 @@ def __is_allowed_to_crud_item_keys(
     object_lists,
     flat_request_body: dict = {},
 ):
-    user_context.bag["soft_call_response_body"] = []
+    user_context.bag["restricted_keys"] = []
     restrictions = restrictions_schema.get("key_restrictions", {})
 
     for restricted_key, restricting_conditions in restrictions.items():
@@ -183,31 +191,54 @@ def __is_allowed_to_crud_item_keys(
         if condition_match:
             if crud == "read":
                 keys_info = interpret_flat_key(restricted_key, object_lists)
-                element = item_in_storage_format
                 for info in keys_info:
-                    if info["is_object_list"]:
+                    if info["object_list"]:
                         element = __get_element_from_object_list_of_item(
-                            element,
+                            item_in_storage_format,
                             info["key"],
                             info["object_key"],
                             object_lists,
                         )
-                element[info["key"]] = "[protected content]"  # pyright: ignore
+                        if element:
+                            item_in_storage_format[info["key"]].remove(element)
+                        break
+                else:
+                    try:
+                        del item_in_storage_format[keys_info[0]["key"]][
+                            keys_info[1]["key"]
+                        ]
+                    except KeyError:
+                        pass
             else:
                 if flat_request_body.get(restricted_key):
-                    user_context.bag["soft_call_response_body"].append(restricted_key)
+                    user_context.bag["restricted_keys"].append(restricted_key)
 
     user_context.bag["requested_item"] = item_in_storage_format
-    return len(user_context.bag["soft_call_response_body"]) == 0
+    return len(user_context.bag["restricted_keys"]) == 0
 
 
 def __item_value_in_values(flat_item, key, values: list, flat_request_body: dict = {}):
     negate_condition = False
+    is_optional = False
+
     if key[0] == "!":
         key = key[1:]
         negate_condition = True
+    if key[0] == "?":
+        key = key[1:]
+        is_optional = True
+
+    try:
+        item_value = flat_request_body.get(key, flat_item[key])
+        if is_optional:
+            negate_condition = False
+    except KeyError:
+        if not is_optional:
+            raise Exception(
+                f"{get_error_code(ErrorCode.METADATA_KEY_UNDEFINED, get_read())} | key:{key} | document:{flat_item.get('_id', flat_item["type"])} - Key {key} not found in document {flat_item.get('_id', flat_item["type"])}. Either prefix the key with '?' in your permission configuration to make it an optional restriction, or patch the document to include the key. '?' will allow access if key does not exist, '!?' will deny access if key does not exist."
+            )
+        return not negate_condition
 
-    item_value = flat_request_body.get(key, flat_item[key])
     expected_values = []
     for value in values:
         if flat_item_key_value := flat_item.get(value):
