@@ -1,11 +1,45 @@
 import json
 import mimetypes
+import re as regex
 
 from cloudevents.conversion import to_dict
 from cloudevents.http import CloudEvent
 from collections.abc import MutableMapping, Iterable
 from copy import deepcopy
 from datetime import datetime, timezone
+from os import getenv
+
+
+URL_UNFRIENDLY_CHARS = {
+    " ": "%20",
+    "!": "%21",
+    "'": "%22",
+    "#": "%23",
+    "%": "%25",
+    "&": "%26",
+    '"': "%27",
+    "(": "%28",
+    ")": "%29",
+    "*": "%2A",
+    "+": "%2B",
+    ",": "%2C",
+    "/": "%2F",
+    ";": "%3B",
+    "<": "%3C",
+    "=": "%3D",
+    ">": "%3E",
+    "?": "%3F",
+    "@": "%40",
+    "[": "%5B",
+    "\\": "%5C",
+    "]": "%5D",
+    "^": "%5E",
+    "`": "%60",
+    "{": "%7B",
+    "|": "%7C",
+    "}": "%7D",
+    "~": "%7E",
+}
 
 
 class CustomJSONEncoder(json.JSONEncoder):
@@ -62,6 +96,7 @@ def __flatten_dict_generator(object_lists, data: MutableMapping, parent_key):
             yield from flatten_dict(object_lists, value, flattened_key).items()
         elif isinstance(value, Iterable) and not isinstance(value, (str, bytes)):
             if all(isinstance(item, MutableMapping) for item in value):
+                item_key = None
                 for item in value:
                     item_key = item.get(object_lists.get(key))
                     if item_key:
@@ -71,6 +106,8 @@ def __flatten_dict_generator(object_lists, data: MutableMapping, parent_key):
                             ).items()
                         else:
                             yield f"{flattened_key}.{item_key}", item["value"]
+                if not item_key:
+                    yield flattened_key, value
             else:
                 yield flattened_key, value
         else:
@@ -97,23 +134,27 @@ def interpret_flat_key(flat_key: str, object_lists):
     keys_info = []
     index = 0
 
-    flat_key_parts = flat_key.split(".")
+    flat_key_parts = regex.split(r"\.(?=(?:[^`]*`[^`]*`)*[^`]*$)", flat_key)
     while index < len(flat_key_parts):
         info = {
             "key": flat_key_parts[index],
-            "is_object_list": flat_key_parts[index] in object_lists.keys(),
+            "object_list": (
+                flat_key_parts[index]
+                if flat_key_parts[index] in object_lists.keys()
+                else ""
+            ),
         }
 
-        if info["is_object_list"]:
+        if info["object_list"]:
             combined_key = "".join(
-                [f"{info['key']}." for info in keys_info if not info["is_object_list"]]
+                [f"{info['key']}." for info in keys_info if not info["object_list"]]
             )
             info["key"] = f"{combined_key}{info['key']}"
             info["object_key"] = flat_key_parts[index + 1]
-            keys_info = [info for info in keys_info if info["is_object_list"]]
+            keys_info = [info for info in keys_info if info["object_list"]]
 
         keys_info.append(info)
-        index += 2 if info["is_object_list"] else 1
+        index += 2 if info["object_list"] else 1
 
     return keys_info
 
@@ -128,6 +169,16 @@ def mediafile_is_public(mediafile):
     ] or copyright_color.lower() in ["green", "groen"]
 
 
+def parse_string_to_bool(value):
+    if isinstance(value, str):
+        value_lower = value.strip().lower()
+        if value_lower in ["true", "yes", "1", "y"]:
+            return True
+        elif value_lower in ["false", "no", "0", "n"]:
+            return False
+    return value
+
+
 def read_json_as_dict(filename, logger):
     try:
         with open(filename) as file:
@@ -137,9 +188,27 @@ def read_json_as_dict(filename, logger):
     return {}
 
 
-def send_cloudevent(mq_client, source, routing_key, data):
+def parse_url_unfriendly_string(
+    input: str, *, replace_char=None, return_unfriendly_chars=False
+):
+    unfriendly_chars = []
+    result = input
+    for char, encoded in URL_UNFRIENDLY_CHARS.items():
+        if char in input:
+            unfriendly_chars.append(char)
+            replacement = encoded if replace_char is None else replace_char
+            result = result.replace(char, replacement)
+    if return_unfriendly_chars:
+        return result, unfriendly_chars
+    return result
+
+
+def send_cloudevent(mq_client, source, routing_key, data, exchange_name=None):
     event = to_dict(CloudEvent({"source": source, "type": routing_key}, data))
-    mq_client.send(event, routing_key=routing_key)
+    if getenv("AMQP_MANAGER", "amqpstorm_flask") in ["amqpstorm_flask"]:
+        mq_client.send(event, routing_key=routing_key, exchange_name=exchange_name)
+    else:
+        mq_client.send(event, routing_key=routing_key)
 
 
 def signal_child_relation_changed(mq_client, collection, id):
@@ -154,10 +223,11 @@ def signal_edge_changed(mq_client, parent_ids_from_changed_edges):
     send_cloudevent(mq_client, "dams", "dams.edge_changed", data)
 
 
-def signal_entity_changed(mq_client, entity):
+def signal_entity_changed(mq_client, entity, unchanged_entity=None):
     data = {
         "location": f"/entities/{get_raw_id(entity)}",
         "type": entity.get("type", "unspecified"),
+        "unchanged_entity": unchanged_entity,
     }
     send_cloudevent(mq_client, "dams", "dams.entity_changed", data)
 
@@ -172,6 +242,11 @@ def signal_mediafiles_added_for_entity(mq_client, entity, mediafiles):
     send_cloudevent(mq_client, "dams", "dams.mediafiles_added_for_entity", data)
 
 
+def signal_relations_deleted_for_entity(mq_client, entity, relations):
+    data = {"entity": entity, "relations": relations}
+    send_cloudevent(mq_client, "dams", "dams.relations_deleted_for_entity", data)
+
+
 def signal_mediafile_changed(mq_client, old_mediafile, mediafile):
     data = {"old_mediafile": old_mediafile, "mediafile": mediafile}
     send_cloudevent(mq_client, "dams", "dams.mediafile_changed", data)
